from .base_classes import *
from .support_functions import *


class AdverseReaction(Resource):
    """Records an unexpected reaction suspected to be related to the
    exposure of the reaction subject to a substance.If the element
    is present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, date=None, subject=None, didNotOccurFlag=None, recorder=None, symptom=None, exposure=None):
        self.original_tagname_ = None
        super(AdverseReaction, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.date = date
        self.subject = subject
        self.didNotOccurFlag = didNotOccurFlag
        self.recorder = recorder
        if symptom is None:
            self.symptom = []
        else:
            self.symptom = symptom
        if exposure is None:
            self.exposure = []
        else:
            self.exposure = exposure
    def factory(*args_, **kwargs_):
        if AdverseReaction.subclass:
            return AdverseReaction.subclass(*args_, **kwargs_)
        else:
            return AdverseReaction(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_didNotOccurFlag(self): return self.didNotOccurFlag
    def set_didNotOccurFlag(self, didNotOccurFlag): self.didNotOccurFlag = didNotOccurFlag
    def get_recorder(self): return self.recorder
    def set_recorder(self, recorder): self.recorder = recorder
    def get_symptom(self): return self.symptom
    def set_symptom(self, symptom): self.symptom = symptom
    def add_symptom(self, value): self.symptom.append(value)
    def insert_symptom(self, index, value): self.symptom[index] = value
    def get_exposure(self): return self.exposure
    def set_exposure(self, exposure): self.exposure = exposure
    def add_exposure(self, value): self.exposure.append(value)
    def insert_exposure(self, index, value): self.exposure[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.date is not None or
            self.subject is not None or
            self.didNotOccurFlag is not None or
            self.recorder is not None or
            self.symptom or
            self.exposure or
            super(AdverseReaction, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='AdverseReaction', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='AdverseReaction')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='AdverseReaction', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='AdverseReaction'):
        super(AdverseReaction, self).exportAttributes(outfile, level, already_processed, namespace_, name_='AdverseReaction')
    def exportChildren(self, outfile, level, namespace_='', name_='AdverseReaction', fromsubclass_=False, pretty_print=True):
        super(AdverseReaction, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.didNotOccurFlag is not None:
            self.didNotOccurFlag.export(outfile, level, namespace_, name_='didNotOccurFlag', pretty_print=pretty_print)
        if self.recorder is not None:
            self.recorder.export(outfile, level, namespace_, name_='recorder', pretty_print=pretty_print)
        for symptom_ in self.symptom:
            symptom_.export(outfile, level, namespace_, name_='symptom', pretty_print=pretty_print)
        for exposure_ in self.exposure:
            exposure_.export(outfile, level, namespace_, name_='exposure', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='AdverseReaction', mapping_=None):
        element = super(AdverseReaction, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.didNotOccurFlag is not None:
            didNotOccurFlag_ = self.didNotOccurFlag
            didNotOccurFlag_.to_etree(element, name_='didNotOccurFlag', mapping_=mapping_)
        if self.recorder is not None:
            recorder_ = self.recorder
            recorder_.to_etree(element, name_='recorder', mapping_=mapping_)
        for symptom_ in self.symptom:
            symptom_.to_etree(element, name_='symptom', mapping_=mapping_)
        for exposure_ in self.exposure:
            exposure_.to_etree(element, name_='exposure', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(AdverseReaction, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'didNotOccurFlag':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.didNotOccurFlag = obj_
            obj_.original_tagname_ = 'didNotOccurFlag'
        elif nodeName_ == 'recorder':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.recorder = obj_
            obj_.original_tagname_ = 'recorder'
        elif nodeName_ == 'symptom':
            obj_ = AdverseReaction_Symptom.factory()
            obj_.build(child_)
            self.symptom.append(obj_)
            obj_.original_tagname_ = 'symptom'
        elif nodeName_ == 'exposure':
            obj_ = AdverseReaction_Exposure.factory()
            obj_.build(child_)
            self.exposure.append(obj_)
            obj_.original_tagname_ = 'exposure'
        super(AdverseReaction, self).buildChildren(child_, node, nodeName_, True)
# end class AdverseReaction


class AdverseReaction_Symptom(BackboneElement):
    """Records an unexpected reaction suspected to be related to the
    exposure of the reaction subject to a substance."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, code=None, severity=None):
        self.original_tagname_ = None
        super(AdverseReaction_Symptom, self).__init__(id, extension, modifierExtension, )
        self.code = code
        self.severity = severity
    def factory(*args_, **kwargs_):
        if AdverseReaction_Symptom.subclass:
            return AdverseReaction_Symptom.subclass(*args_, **kwargs_)
        else:
            return AdverseReaction_Symptom(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_severity(self): return self.severity
    def set_severity(self, severity): self.severity = severity
    def hasContent_(self):
        if (
            self.code is not None or
            self.severity is not None or
            super(AdverseReaction_Symptom, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='AdverseReaction.Symptom', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='AdverseReaction.Symptom')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='AdverseReaction.Symptom', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='AdverseReaction.Symptom'):
        super(AdverseReaction_Symptom, self).exportAttributes(outfile, level, already_processed, namespace_, name_='AdverseReaction.Symptom')
    def exportChildren(self, outfile, level, namespace_='', name_='AdverseReaction.Symptom', fromsubclass_=False, pretty_print=True):
        super(AdverseReaction_Symptom, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.severity is not None:
            self.severity.export(outfile, level, namespace_, name_='severity', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='AdverseReaction.Symptom', mapping_=None):
        element = super(AdverseReaction_Symptom, self).to_etree(parent_element, name_, mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.severity is not None:
            severity_ = self.severity
            severity_.to_etree(element, name_='severity', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(AdverseReaction_Symptom, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'severity':
            obj_ = ReactionSeverity.factory()
            obj_.build(child_)
            self.severity = obj_
            obj_.original_tagname_ = 'severity'
        super(AdverseReaction_Symptom, self).buildChildren(child_, node, nodeName_, True)
# end class AdverseReaction_Symptom


class AdverseReaction_Exposure(BackboneElement):
    """Records an unexpected reaction suspected to be related to the
    exposure of the reaction subject to a substance."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, date=None, type_=None, causalityExpectation=None, substance=None):
        self.original_tagname_ = None
        super(AdverseReaction_Exposure, self).__init__(id, extension, modifierExtension, )
        self.date = date
        self.type_ = type_
        self.causalityExpectation = causalityExpectation
        self.substance = substance
    def factory(*args_, **kwargs_):
        if AdverseReaction_Exposure.subclass:
            return AdverseReaction_Exposure.subclass(*args_, **kwargs_)
        else:
            return AdverseReaction_Exposure(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_causalityExpectation(self): return self.causalityExpectation
    def set_causalityExpectation(self, causalityExpectation): self.causalityExpectation = causalityExpectation
    def get_substance(self): return self.substance
    def set_substance(self, substance): self.substance = substance
    def hasContent_(self):
        if (
            self.date is not None or
            self.type_ is not None or
            self.causalityExpectation is not None or
            self.substance is not None or
            super(AdverseReaction_Exposure, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='AdverseReaction.Exposure', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='AdverseReaction.Exposure')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='AdverseReaction.Exposure', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='AdverseReaction.Exposure'):
        super(AdverseReaction_Exposure, self).exportAttributes(outfile, level, already_processed, namespace_, name_='AdverseReaction.Exposure')
    def exportChildren(self, outfile, level, namespace_='', name_='AdverseReaction.Exposure', fromsubclass_=False, pretty_print=True):
        super(AdverseReaction_Exposure, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.causalityExpectation is not None:
            self.causalityExpectation.export(outfile, level, namespace_, name_='causalityExpectation', pretty_print=pretty_print)
        if self.substance is not None:
            self.substance.export(outfile, level, namespace_, name_='substance', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='AdverseReaction.Exposure', mapping_=None):
        element = super(AdverseReaction_Exposure, self).to_etree(parent_element, name_, mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.causalityExpectation is not None:
            causalityExpectation_ = self.causalityExpectation
            causalityExpectation_.to_etree(element, name_='causalityExpectation', mapping_=mapping_)
        if self.substance is not None:
            substance_ = self.substance
            substance_.to_etree(element, name_='substance', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(AdverseReaction_Exposure, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'type':
            obj_ = ExposureType.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'causalityExpectation':
            obj_ = CausalityExpectation.factory()
            obj_.build(child_)
            self.causalityExpectation = obj_
            obj_.original_tagname_ = 'causalityExpectation'
        elif nodeName_ == 'substance':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.substance = obj_
            obj_.original_tagname_ = 'substance'
        super(AdverseReaction_Exposure, self).buildChildren(child_, node, nodeName_, True)
# end class AdverseReaction_Exposure


class ReactionSeverity(Element):
    """The severity of an adverse reaction.If the element is present, it
    must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ReactionSeverity, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ReactionSeverity.subclass:
            return ReactionSeverity.subclass(*args_, **kwargs_)
        else:
            return ReactionSeverity(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ReactionSeverity, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ReactionSeverity', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ReactionSeverity')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ReactionSeverity', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ReactionSeverity'):
        super(ReactionSeverity, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ReactionSeverity')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ReactionSeverity', fromsubclass_=False, pretty_print=True):
        super(ReactionSeverity, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ReactionSeverity', mapping_=None):
        element = super(ReactionSeverity, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ReactionSeverity, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ReactionSeverity, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ReactionSeverity


class ExposureType(Element):
    """The type of exposure that resulted in an adverse reactionIf the
    element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ExposureType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ExposureType.subclass:
            return ExposureType.subclass(*args_, **kwargs_)
        else:
            return ExposureType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ExposureType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ExposureType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ExposureType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ExposureType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ExposureType'):
        super(ExposureType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ExposureType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ExposureType', fromsubclass_=False, pretty_print=True):
        super(ExposureType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ExposureType', mapping_=None):
        element = super(ExposureType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ExposureType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ExposureType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ExposureType


class CausalityExpectation(Element):
    """How likely is it that the given exposure caused a reactionIf the
    element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(CausalityExpectation, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if CausalityExpectation.subclass:
            return CausalityExpectation.subclass(*args_, **kwargs_)
        else:
            return CausalityExpectation(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(CausalityExpectation, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CausalityExpectation', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CausalityExpectation')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CausalityExpectation', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CausalityExpectation'):
        super(CausalityExpectation, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CausalityExpectation')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='CausalityExpectation', fromsubclass_=False, pretty_print=True):
        super(CausalityExpectation, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CausalityExpectation', mapping_=None):
        element = super(CausalityExpectation, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(CausalityExpectation, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(CausalityExpectation, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class CausalityExpectation


class SensitivityStatus(Element):
    """The status of the adverse sensitivityIf the element is present, it
    must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(SensitivityStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if SensitivityStatus.subclass:
            return SensitivityStatus.subclass(*args_, **kwargs_)
        else:
            return SensitivityStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(SensitivityStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='SensitivityStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='SensitivityStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='SensitivityStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='SensitivityStatus'):
        super(SensitivityStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='SensitivityStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='SensitivityStatus', fromsubclass_=False, pretty_print=True):
        super(SensitivityStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='SensitivityStatus', mapping_=None):
        element = super(SensitivityStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(SensitivityStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(SensitivityStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class SensitivityStatus


class Criticality(Element):
    """The criticality of an adverse sensitivityIf the element is present,
    it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(Criticality, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if Criticality.subclass:
            return Criticality.subclass(*args_, **kwargs_)
        else:
            return Criticality(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(Criticality, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Criticality', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Criticality')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Criticality', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Criticality'):
        super(Criticality, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Criticality')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='Criticality', fromsubclass_=False, pretty_print=True):
        super(Criticality, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Criticality', mapping_=None):
        element = super(Criticality, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(Criticality, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(Criticality, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class Criticality


class SensitivityType(Element):
    """The type of an adverse sensitivityIf the element is present, it must
    have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(SensitivityType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if SensitivityType.subclass:
            return SensitivityType.subclass(*args_, **kwargs_)
        else:
            return SensitivityType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(SensitivityType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='SensitivityType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='SensitivityType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='SensitivityType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='SensitivityType'):
        super(SensitivityType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='SensitivityType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='SensitivityType', fromsubclass_=False, pretty_print=True):
        super(SensitivityType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='SensitivityType', mapping_=None):
        element = super(SensitivityType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(SensitivityType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(SensitivityType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class SensitivityType


class AllergyIntolerance(Resource):
    """Indicates the patient has a susceptibility to an adverse reaction
    upon exposure to a specified substance.If the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, criticality=None, sensitivityType=None, recordedDate=None, status=None, subject=None, recorder=None, substance=None, reaction=None, sensitivityTest=None):
        self.original_tagname_ = None
        super(AllergyIntolerance, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.criticality = criticality
        self.sensitivityType = sensitivityType
        self.recordedDate = recordedDate
        self.status = status
        self.subject = subject
        self.recorder = recorder
        self.substance = substance
        if reaction is None:
            self.reaction = []
        else:
            self.reaction = reaction
        if sensitivityTest is None:
            self.sensitivityTest = []
        else:
            self.sensitivityTest = sensitivityTest
    def factory(*args_, **kwargs_):
        if AllergyIntolerance.subclass:
            return AllergyIntolerance.subclass(*args_, **kwargs_)
        else:
            return AllergyIntolerance(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_criticality(self): return self.criticality
    def set_criticality(self, criticality): self.criticality = criticality
    def get_sensitivityType(self): return self.sensitivityType
    def set_sensitivityType(self, sensitivityType): self.sensitivityType = sensitivityType
    def get_recordedDate(self): return self.recordedDate
    def set_recordedDate(self, recordedDate): self.recordedDate = recordedDate
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_recorder(self): return self.recorder
    def set_recorder(self, recorder): self.recorder = recorder
    def get_substance(self): return self.substance
    def set_substance(self, substance): self.substance = substance
    def get_reaction(self): return self.reaction
    def set_reaction(self, reaction): self.reaction = reaction
    def add_reaction(self, value): self.reaction.append(value)
    def insert_reaction(self, index, value): self.reaction[index] = value
    def get_sensitivityTest(self): return self.sensitivityTest
    def set_sensitivityTest(self, sensitivityTest): self.sensitivityTest = sensitivityTest
    def add_sensitivityTest(self, value): self.sensitivityTest.append(value)
    def insert_sensitivityTest(self, index, value): self.sensitivityTest[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.criticality is not None or
            self.sensitivityType is not None or
            self.recordedDate is not None or
            self.status is not None or
            self.subject is not None or
            self.recorder is not None or
            self.substance is not None or
            self.reaction or
            self.sensitivityTest or
            super(AllergyIntolerance, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='AllergyIntolerance', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='AllergyIntolerance')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='AllergyIntolerance', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='AllergyIntolerance'):
        super(AllergyIntolerance, self).exportAttributes(outfile, level, already_processed, namespace_, name_='AllergyIntolerance')
    def exportChildren(self, outfile, level, namespace_='', name_='AllergyIntolerance', fromsubclass_=False, pretty_print=True):
        super(AllergyIntolerance, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.criticality is not None:
            self.criticality.export(outfile, level, namespace_, name_='criticality', pretty_print=pretty_print)
        if self.sensitivityType is not None:
            self.sensitivityType.export(outfile, level, namespace_, name_='sensitivityType', pretty_print=pretty_print)
        if self.recordedDate is not None:
            self.recordedDate.export(outfile, level, namespace_, name_='recordedDate', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.recorder is not None:
            self.recorder.export(outfile, level, namespace_, name_='recorder', pretty_print=pretty_print)
        if self.substance is not None:
            self.substance.export(outfile, level, namespace_, name_='substance', pretty_print=pretty_print)
        for reaction_ in self.reaction:
            reaction_.export(outfile, level, namespace_, name_='reaction', pretty_print=pretty_print)
        for sensitivityTest_ in self.sensitivityTest:
            sensitivityTest_.export(outfile, level, namespace_, name_='sensitivityTest', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='AllergyIntolerance', mapping_=None):
        element = super(AllergyIntolerance, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.criticality is not None:
            criticality_ = self.criticality
            criticality_.to_etree(element, name_='criticality', mapping_=mapping_)
        if self.sensitivityType is not None:
            sensitivityType_ = self.sensitivityType
            sensitivityType_.to_etree(element, name_='sensitivityType', mapping_=mapping_)
        if self.recordedDate is not None:
            recordedDate_ = self.recordedDate
            recordedDate_.to_etree(element, name_='recordedDate', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.recorder is not None:
            recorder_ = self.recorder
            recorder_.to_etree(element, name_='recorder', mapping_=mapping_)
        if self.substance is not None:
            substance_ = self.substance
            substance_.to_etree(element, name_='substance', mapping_=mapping_)
        for reaction_ in self.reaction:
            reaction_.to_etree(element, name_='reaction', mapping_=mapping_)
        for sensitivityTest_ in self.sensitivityTest:
            sensitivityTest_.to_etree(element, name_='sensitivityTest', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(AllergyIntolerance, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'criticality':
            obj_ = Criticality.factory()
            obj_.build(child_)
            self.criticality = obj_
            obj_.original_tagname_ = 'criticality'
        elif nodeName_ == 'sensitivityType':
            obj_ = SensitivityType.factory()
            obj_.build(child_)
            self.sensitivityType = obj_
            obj_.original_tagname_ = 'sensitivityType'
        elif nodeName_ == 'recordedDate':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.recordedDate = obj_
            obj_.original_tagname_ = 'recordedDate'
        elif nodeName_ == 'status':
            obj_ = SensitivityStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'recorder':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.recorder = obj_
            obj_.original_tagname_ = 'recorder'
        elif nodeName_ == 'substance':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.substance = obj_
            obj_.original_tagname_ = 'substance'
        elif nodeName_ == 'reaction':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.reaction.append(obj_)
            obj_.original_tagname_ = 'reaction'
        elif nodeName_ == 'sensitivityTest':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.sensitivityTest.append(obj_)
            obj_.original_tagname_ = 'sensitivityTest'
        super(AllergyIntolerance, self).buildChildren(child_, node, nodeName_, True)
# end class AllergyIntolerance



import unittest
import trytond.tests.test_tryton
from trytond.tests.test_tryton import ModuleTestCase


class HealthICD10TestCase(ModuleTestCase):
    '''
    Test Health ICD10 module.
    '''
    module = 'health_icd10'


def suite():
    suite = trytond.tests.test_tryton.suite()
    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
        HealthICD10TestCase))
    return suite



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields
from datetime import datetime
from dateutil.relativedelta import relativedelta
from trytond.transaction import Transaction
from trytond.pyson import Eval, Not, Bool, Equal


__all__ = ['InpatientRegistration', 'InpatientIcu', 'Glasgow', 'ApacheII',
            'MechanicalVentilation', 'ChestDrainageAssessment',
            'PatientRounding']


class InpatientRegistration(ModelSQL, ModelView):
    'Patient admission History'
    __name__ = 'gnuhealth.inpatient.registration'
    icu = fields.Boolean('ICU', help='Shows if patient was admitted to'
        ' the Intensive Care Unit during the hospitalization period')
    icu_admissions = fields.One2Many('gnuhealth.inpatient.icu',
        'name', "ICU Admissions")


class InpatientIcu(ModelSQL, ModelView):
    'Patient ICU Information'
    __name__ = 'gnuhealth.inpatient.icu'

    def icu_duration(self, name):
        if self.discharged_from_icu:
            end = self.icu_discharge_date
        else:
            end = datetime.now()
        return end.date() - self.icu_admission_date.date()

    name = fields.Many2One('gnuhealth.inpatient.registration',
        'Registration Code', required=True)

    admitted = fields.Boolean('Admitted', help="Will be set when the patient \
        is currently admitted at ICU")

    icu_admission_date = fields.DateTime('ICU Admission',
        help="ICU Admission Date", required=True)
    discharged_from_icu = fields.Boolean('Discharged')
    icu_discharge_date = fields.DateTime('Discharge', states={
            'invisible': Not(Bool(Eval('discharged_from_icu'))),
            'required': Bool(Eval('discharged_from_icu')),
            },
        depends=['discharged_from_icu'])
    icu_stay = fields.Function(fields.TimeDelta('Duration'), 'icu_duration')

    mv_history = fields.One2Many('gnuhealth.icu.ventilation',
        'name', "Mechanical Ventilation History")

    @classmethod
    def __setup__(cls):
        super(InpatientIcu, cls).__setup__()
        cls._error_messages.update({
            'patient_already_at_icu': 'Our records indicate that the patient'
                ' is already admitted at ICU'})

    @classmethod
    def validate(cls, inpatients):
        super(InpatientIcu, cls).validate(inpatients)
        for inpatient in inpatients:
            inpatient.check_patient_admitted_at_icu()

    def check_patient_admitted_at_icu(self):
        # Verify that the patient is not at ICU already
        cursor = Transaction().connection.cursor()
        cursor.execute("SELECT count(name) "
            "FROM " + self._table + "  \
            WHERE (name = %s AND admitted)",
            (str(self.name.id),))
        if cursor.fetchone()[0] > 1:
            self.raise_user_error('patient_already_at_icu')

    @staticmethod
    def default_admitted():
        return True

    @fields.depends('discharged_from_icu')
    def on_change_with_admitted(self):
        # Reset the admission flag when the patient is discharged from ICU
        if self.discharged_from_icu:
            res = False
        else:
            res = True
        return res


class Glasgow(ModelSQL, ModelView):
    'Glasgow Coma Scale'
    __name__ = 'gnuhealth.icu.glasgow'

    name = fields.Many2One('gnuhealth.inpatient.registration',
        'Registration Code', required=True)

    evaluation_date = fields.DateTime('Date', help="Date / Time",
        required=True)

    glasgow = fields.Integer('Glasgow',
        help='Level of Consciousness - on Glasgow Coma Scale :  < 9 severe -'
        ' 9-12 Moderate, > 13 minor')
    glasgow_eyes = fields.Selection([
        ('1', '1 : Does not Open Eyes'),
        ('2', '2 : Opens eyes in response to painful stimuli'),
        ('3', '3 : Opens eyes in response to voice'),
        ('4', '4 : Opens eyes spontaneously'),
        ], 'Eyes', sort=False)
    glasgow_verbal = fields.Selection([
        ('1', '1 : Makes no sounds'),
        ('2', '2 : Incomprehensible sounds'),
        ('3', '3 : Utters inappropriate words'),
        ('4', '4 : Confused, disoriented'),
        ('5', '5 : Oriented, converses normally'),
        ], 'Verbal', sort=False)
    glasgow_motor = fields.Selection([
        ('1', '1 : Makes no movement'),
        ('2', '2 : Extension to painful stimuli - decerebrate response -'),
        ('3', '3 : Abnormal flexion to painful stimuli \
            (decorticate response)'),
        ('4', '4 : Flexion / Withdrawal to painful stimuli'),
        ('5', '5 : localizes painful stimuli'),
        ('6', '6 : Obeys commands'),
        ], 'Motor', sort=False)

    @staticmethod
    def default_glasgow_eyes():
        return '4'

    @staticmethod
    def default_glasgow_verbal():
        return '5'

    @staticmethod
    def default_glasgow_motor():
        return '6'

    @staticmethod
    def default_glasgow():
        return 15

    # Default evaluation date
    @staticmethod
    def default_evaluation_date():
        return datetime.now()

    @fields.depends('glasgow_verbal', 'glasgow_motor', 'glasgow_eyes')
    def on_change_with_glasgow(self):
        return int(self.glasgow_motor) + int(self.glasgow_eyes) + \
            int(self.glasgow_verbal)

    # Return the Glasgow Score with each component
    def get_rec_name(self, name):
        if self.name:
            res = str(self.glasgow) + ': ' + 'E' + self.glasgow_eyes + ' V' + \
                self.glasgow_verbal + ' M' + self.glasgow_motor
        return res


class ApacheII(ModelSQL, ModelView):
    'Apache II scoring'
    __name__ = 'gnuhealth.icu.apache2'

    name = fields.Many2One('gnuhealth.inpatient.registration',
        'Registration Code', required=True)
    score_date = fields.DateTime('Date', help="Date of the score",
        required=True)

    age = fields.Integer('Age', help='Patient age in years')
    temperature = fields.Float('Temperature', help='Rectal temperature')
    mean_ap = fields.Integer('MAP', help='Mean Arterial Pressure')
    heart_rate = fields.Integer('Heart Rate')
    respiratory_rate = fields.Integer('Respiratory Rate')
    fio2 = fields.Float('FiO2')
    pao2 = fields.Integer('PaO2')
    paco2 = fields.Integer('PaCO2')
    aado2 = fields.Integer('A-a DO2')

    ph = fields.Float('pH')
    serum_sodium = fields.Integer('Sodium')
    serum_potassium = fields.Float('Potassium')
    serum_creatinine = fields.Float('Creatinine')
    arf = fields.Boolean('ARF', help='Acute Renal Failure')
    wbc = fields.Float('WBC', help="White blood cells x 1000 - if you"
        " want to input 4500 wbc / ml, type in 4.5")
    hematocrit = fields.Float('Hematocrit')
    gcs = fields.Integer('GSC', help='Last Glasgow Coma Scale'
        ' You can use the GSC calculator from the Patient Evaluation Form.')
    chronic_condition = fields.Boolean('Chronic condition',
        help='Organ Failure or immunocompromised patient')
    hospital_admission_type = fields.Selection([
        (None, ''),
        ('me', 'Medical or emergency postoperative'),
        ('el', 'elective postoperative')],
        'Hospital Admission Type', states={
            'invisible': Not(Bool(Eval('chronic_condition'))),
            'required': Bool(Eval('chronic_condition'))}, sort=False)

    apache_score = fields.Integer('Score')

    #Default FiO2 PaO2 and PaCO2 so we do the A-a gradient
    #calculation with non-null values

    @fields.depends('fio2', 'pao2', 'paco2')
    def on_change_with_aado2(self):
    # Calculates the Alveolar-arterial difference
    # based on FiO2, PaCO2 and PaO2 values
        if (self.fio2 and self.paco2 and self.pao2):
            return (713 * self.fio2) - (self.paco2 / 0.8) - self.pao2

    @fields.depends('age', 'temperature', 'mean_ap', 'heart_rate',
        'respiratory_rate', 'fio2', 'pao2', 'aado2', 'ph', 'serum_sodium',
        'serum_potassium', 'serum_creatinine', 'arf', 'wbc', 'hematocrit',
        'gcs', 'chronic_condition', 'hospital_admission_type')
    def on_change_with_apache_score(self):
    # Calculate the APACHE SCORE from the variables in the

        total = 0
        # Age
        if (self.age):
            if (self.age > 44 and self.age < 55):
                total = total + 2
            elif (self.age > 54 and self.age < 65):
                total = total + 3
            elif (self.age > 64 and self.age < 75):
                total = total + 5
            elif (self.age > 74):
                total = total + 6

        # Temperature
        if (self.temperature):
            if ((self.temperature >= 38.5 and self.temperature < 39) or
                (self.temperature >= 34 and self.temperature < 36)):
                    total = total + 1
            elif (self.temperature >= 32 and self.temperature < 34):
                total = total + 2
            elif ((self.temperature >= 30 and self.temperature < 32) or
                (self.temperature >= 39 and self.temperature < 41)):
                total = total + 3
            elif (self.temperature >= 41 or self.temperature < 30):
                total = total + 4

        # Mean Arterial Pressure (MAP)
        if (self.mean_ap):
            if ((self.mean_ap >= 110 and self.mean_ap < 130) or
                (self.mean_ap >= 50 and self.mean_ap < 70)):
                    total = total + 2
            elif (self.mean_ap >= 130 and self.mean_ap < 160):
                total = total + 3
            elif (self.mean_ap >= 160 or self.mean_ap < 50):
                total = total + 4

        # Heart Rate
        if (self.heart_rate):
            if ((self.heart_rate >= 55 and self.heart_rate < 70) or
                (self.heart_rate >= 110 and self.heart_rate < 140)):
                    total = total + 2
            elif ((self.heart_rate >= 40 and self.heart_rate < 55) or
                (self.heart_rate >= 140 and self.heart_rate < 180)):
                    total = total + 3
            elif (self.heart_rate >= 180 or self.heart_rate < 40):
                total = total + 4

        # Respiratory Rate
        if (self.respiratory_rate):
            if ((self.respiratory_rate >= 10 and self.respiratory_rate < 12) or
                (self.respiratory_rate >= 25 and self.respiratory_rate < 35)):
                    total = total + 1
            elif (self.respiratory_rate >= 6 and self.respiratory_rate < 10):
                    total = total + 2
            elif (self.respiratory_rate >= 35 and self.respiratory_rate < 50):
                    total = total + 3
            elif (self.respiratory_rate >= 50 or self.respiratory_rate < 6):
                total = total + 4

        # FIO2
        if (self.fio2):
            # If Fi02 is greater than 0.5, we measure the AaDO2 gradient
            # Otherwise, we take into account the Pa02 value

            if (self.fio2 >= 0.5):
                if (self.aado2 >= 200 and self.aado2 < 350):
                    total = total + 2

                elif (self.aado2 >= 350 and self.aado2 < 500):
                    total = total + 3

                elif (self.aado2 >= 500):
                    total = total + 4

            else:
                if (self.pao2 >= 61 and self.pao2 < 71):
                    total = total + 1

                elif (self.pao2 >= 55 and self.pao2 < 61):
                    total = total + 3

                elif (self.pao2 < 55):
                    total = total + 4

        # Arterial pH
        if (self.ph):
            if (self.ph >= 7.5 and self.ph < 7.6):
                    total = total + 1
            elif (self.ph >= 7.25 and self.ph < 7.33):
                    total = total + 2
            elif ((self.ph >= 7.15 and self.ph < 7.25) or
                (self.ph >= 7.6 and self.ph < 7.7)):
                    total = total + 3
            elif (self.ph >= 7.7 or self.ph < 7.15):
                total = total + 4

        # Serum Sodium
        if (self.serum_sodium):
            if (self.serum_sodium >= 150 and self.serum_sodium < 155):
                    total = total + 1
            elif ((self.serum_sodium >= 155 and self.serum_sodium < 160) or
                (self.serum_sodium >= 120 and self.serum_sodium < 130)):
                    total = total + 2
            elif ((self.serum_sodium >= 160 and self.serum_sodium < 180) or
                (self.serum_sodium >= 111 and self.serum_sodium < 120)):
                    total = total + 3
            elif (self.serum_sodium >= 180 or self.serum_sodium < 111):
                total = total + 4

        # Serum Potassium
        if (self.serum_potassium):
            if ((self.serum_potassium >= 3 and self.serum_potassium < 3.5) or
                (self.serum_potassium >= 5.5 and self.serum_potassium < 6)):
                    total = total + 1
            elif (self.serum_potassium >= 2.5 and self.serum_potassium < 3):
                total = total + 2
            elif (self.serum_potassium >= 6 and self.serum_potassium < 7):
                total = total + 3
            elif (self.serum_potassium >= 7 or self.serum_potassium < 2.5):
                total = total + 4

        # Serum Creatinine
        if (self.serum_creatinine):
            arf_factor = 1
            if (self.arf):
            # We multiply by 2 the score if there is concomitant ARF
                arf_factor = 2
            if ((self.serum_creatinine < 0.6) or
                (self.serum_creatinine >= 1.5 and self.serum_creatinine < 2)):
                    total = total + 2 * arf_factor
            elif (self.serum_creatinine >= 2 and self.serum_creatinine < 3.5):
                total = total + 3 * arf_factor
            elif (self.serum_creatinine >= 3.5):
                total = total + 4 * arf_factor

        # Hematocrit
        if (self.hematocrit):
            if (self.hematocrit >= 46 and self.hematocrit < 50):
                total = total + 1
            elif ((self.hematocrit >= 50 and self.hematocrit < 60) or
                (self.hematocrit >= 20 and self.hematocrit < 30)):
                    total = total + 2
            elif (self.hematocrit >= 60 or self.hematocrit < 20):
                total = total + 4

        # WBC ( x 1000 )
        if (self.wbc):
            if (self.wbc >= 15 and self.wbc < 20):
                total = total + 1
            elif ((self.wbc >= 20 and self.wbc < 40) or
                (self.wbc >= 1 and self.wbc < 3)):
                    total = total + 2
            elif (self.wbc >= 40 or self.wbc < 1):
                total = total + 4

        # Immnunocompromised or severe organ failure
        if (self.chronic_condition):
            if (self.hospital_admission_type == 'me'):
                total = total + 5
            else:
                total = total + 2

        return total


class MechanicalVentilation(ModelSQL, ModelView):
    'Mechanical Ventilation History'
    __name__ = 'gnuhealth.icu.ventilation'

    def mv_duration(self, name):
        start = end = datetime.now()

        if self.mv_start:
            start = self.mv_start

        if self.mv_end:
            end = self.mv_end

        return end.date() - start.date()

    name = fields.Many2One('gnuhealth.inpatient.icu', 'Patient ICU Admission',
        required=True)

    ventilation = fields.Selection([
        (None, ''),
        ('none', 'None - Maintains Own'),
        ('nppv', 'Non-Invasive Positive Pressure'),
        ('ett', 'ETT'),
        ('tracheostomy', 'Tracheostomy')],
        'Type', help="NPPV = Non-Invasive Positive "
            "Pressure Ventilation, BiPAP-CPAP \n"
            "ETT - Endotracheal Tube", sort=False)

    ett_size = fields.Integer('ETT Size', states={
            'invisible': Not(Equal(Eval('ventilation'), 'ett'))})

    tracheostomy_size = fields.Integer('Tracheostomy size', states={
            'invisible': Not(Equal(Eval('ventilation'), 'tracheostomy'))})

    mv_start = fields.DateTime('From', help="Start of Mechanical Ventilation",
        required=True)
    mv_end = fields.DateTime('To', help="End of Mechanical Ventilation",
        states={
            'invisible': Bool(Eval('current_mv')),
            'required': Not(Bool(Eval('current_mv'))),
            },
        depends=['current_mv'])
    mv_period = fields.Function(fields.TimeDelta('Duration'), 'mv_duration')
    current_mv = fields.Boolean('Current')
    remarks = fields.Char('Remarks')

    @classmethod
    def __setup__(cls):
        super(MechanicalVentilation, cls).__setup__()
        cls._error_messages.update({
            'patient_already_on_mv': 'Our records indicate that the patient'
                ' is already on Mechanical Ventilation !'})

    @classmethod
    def validate(cls, inpatients):
        super(MechanicalVentilation, cls).validate(inpatients)
        for inpatient in inpatients:
            inpatient.check_patient_current_mv()

    def check_patient_current_mv(self):
        # Check for only one current mechanical ventilation on patient
        cursor = Transaction().connection.cursor()
        cursor.execute("SELECT count(name) "
            "FROM " + self._table + "  \
            WHERE (name = %s AND current_mv)",
            (str(self.name.id),))
        if cursor.fetchone()[0] > 1:
            self.raise_user_error('patient_already_on_mv')

    @staticmethod
    def default_current_mv():
        return True


class ChestDrainageAssessment(ModelSQL, ModelView):
    'Chest Drainage Asessment'
    __name__ = 'gnuhealth.icu.chest_drainage'

    name = fields.Many2One('gnuhealth.patient.rounding', 'Rounding',
        required=True)
    location = fields.Selection([
        (None, ''),
        ('rl', 'Right Pleura'),
        ('ll', 'Left Pleura'),
        ('mediastinum', 'Mediastinum')],
        'Location', sort=False)
    fluid_aspect = fields.Selection([
        (None, ''),
        ('serous', 'Serous'),
        ('bloody', 'Bloody'),
        ('chylous', 'Chylous'),
        ('purulent', 'Purulent')],
        'Aspect', sort=False)
    suction = fields.Boolean('Suction')
    suction_pressure = fields.Integer('cm H2O', states={
            'invisible': Not(Bool(Eval('suction'))),
            'required': Bool(Eval('suction')),
            },
        depends=['suction'])
    oscillation = fields.Boolean('Oscillation')
    air_leak = fields.Boolean('Air Leak')
    fluid_volume = fields.Integer('Volume')
    remarks = fields.Char('Remarks')


class PatientRounding(ModelSQL, ModelView):
    # Nursing Rounding for ICU
    # Inherit and append to the existing model the new functionality for ICU

    __name__ = 'gnuhealth.patient.rounding'

    STATES = {'readonly': Eval('state') == 'done'}

    icu_patient = fields.Boolean('ICU', help='Check this box if this is'
    'an Intensive Care Unit rounding.', states=STATES)
    # Neurological assesment
    gcs = fields.Many2One('gnuhealth.icu.glasgow', 'GCS',
        domain=[('name', '=', Eval('name'))], depends=['name'],states=STATES)

    pupil_dilation = fields.Selection([
        ('normal', 'Normal'),
        ('miosis', 'Miosis'),
        ('mydriasis', 'Mydriasis')],
        'Pupil Dilation', sort=False, states=STATES)

    left_pupil = fields.Integer('L', help="size in mm of left pupil",
        states=STATES)
    right_pupil = fields.Integer('R', help="size in mm of right pupil",
        states=STATES)

    anisocoria = fields.Boolean('Anisocoria', states=STATES)

    pupillary_reactivity = fields.Selection([
        (None, ''),
        ('brisk', 'Brisk'),
        ('sluggish', 'Sluggish'),
        ('nonreactive', 'Nonreactive')],
        'Pupillary Reactivity', sort=False, states=STATES)

    pupil_consensual_resp = fields.Boolean('Consensual Response',
        help="Pupillary Consensual Response", states=STATES)

    # Respiratory assesment
    # Mechanical ventilation information is on the patient ICU general info

    respiration_type = fields.Selection([
        (None, ''),
        ('regular', 'Regular'),
        ('deep', 'Deep'),
        ('shallow', 'Shallow'),
        ('labored', 'Labored'),
        ('intercostal', 'Intercostal')],
        'Respiration', sort=False, states=STATES)

    oxygen_mask = fields.Boolean('Oxygen Mask', states=STATES)
    fio2 = fields.Integer('FiO2', states=STATES)

    peep = fields.Boolean('PEEP', states=STATES)

    peep_pressure = fields.Integer('cm H2O', help="Pressure", states={
            'invisible': Not(Bool(Eval('peep'))),
            'required': Bool(Eval('peep')),
            'readonly': Eval('state') == 'done',
            },
        depends=['peep'])

    sce = fields.Boolean('SCE', help="Subcutaneous Emphysema",states=STATES)
    lips_lesion = fields.Boolean('Lips lesion',states=STATES)
    oral_mucosa_lesion = fields.Boolean('Oral mucosa lesion', states=STATES)

    # Chest expansion characteristics
    chest_expansion = fields.Selection([
        (None, ''),
        ('symmetric', 'Symmetrical'),
        ('asymmetric', 'Asymmetrical')],
        'Expansion', sort=False, states=STATES)
    paradoxical_expansion = fields.Boolean('Paradoxical',
        help="Paradoxical Chest Expansion", states=STATES)
    tracheal_tug = fields.Boolean('Tracheal Tug', states=STATES)

    # Trachea position
    trachea_alignment = fields.Selection([
        (None, ''),
        ('midline', 'Midline'),
        ('right', 'Deviated right'),
        ('left', 'Deviated left')],
        'Tracheal alignment', sort=False, states=STATES)

    # Chest Drainages
    chest_drainages = fields.One2Many('gnuhealth.icu.chest_drainage',
        'name', "Drainages", states=STATES)

    # Chest X-Ray
    xray = fields.Binary('Xray', states=STATES)

    # Cardiovascular assessment

    ecg = fields.Many2One('gnuhealth.patient.ecg', 'Inpatient ECG',
        domain=[('inpatient_registration_code', '=', Eval('name'))],
        depends=['name'],states=STATES)

    venous_access = fields.Selection([
        (None, ''),
        ('none', 'None'),
        ('central', 'Central catheter'),
        ('peripheral', 'Peripheral')],
        'Venous Access', sort=False, states=STATES)

    swan_ganz = fields.Boolean('Swan Ganz',
        help="Pulmonary Artery Catheterization - PAC -", states=STATES)

    arterial_access = fields.Boolean('Arterial Access', states=STATES)

    dialysis = fields.Boolean('Dialysis', states=STATES)

    edema = fields.Selection([
        (None, ''),
        ('none', 'None'),
        ('peripheral', 'Peripheral'),
        ('anasarca', 'Anasarca')],
        'Edema', sort=False, states=STATES)

    # Blood & Skin
    bacteremia = fields.Boolean('Bacteremia', states=STATES)
    ssi = fields.Boolean('Surgery Site Infection', states=STATES)
    wound_dehiscence = fields.Boolean('Wound Dehiscence', states=STATES)
    cellulitis = fields.Boolean('Cellulitis', states=STATES)
    necrotizing_fasciitis = fields.Boolean('Necrotizing fasciitis',
        states=STATES)

    # Abdomen & Digestive

    vomiting = fields.Selection([
        (None, ''),
        ('none', 'None'),
        ('vomiting', 'Vomiting'),
        ('hematemesis', 'Hematemesis')],
        'Vomiting', sort=False, states=STATES)

    bowel_sounds = fields.Selection([
        (None, ''),
        ('normal', 'Normal'),
        ('increased', 'Increased'),
        ('decreased', 'Decreased'),
        ('absent', 'Absent')],
        'Bowel Sounds', sort=False, states=STATES)

    stools = fields.Selection([
        (None, ''),
        ('normal', 'Normal'),
        ('constipation', 'Constipation'),
        ('diarrhea', 'Diarrhea'),
        ('melena', 'Melena')],
        'Stools', sort=False, states=STATES)

    peritonitis = fields.Boolean('Peritonitis signs', states=STATES)

    @fields.depends('left_pupil', 'right_pupil')
    def on_change_with_anisocoria(self):
        if (self.left_pupil == self.right_pupil):
            return False
        else:
            return True

    @staticmethod
    def default_pupil_dilation():
        return 'normal'
